/***************************************************************************
                           virtualaddressdialog.cpp
                           -------------------------------
    copyright            : (C) 2010 Guido Scholz
    e-mail               : guido.scholz@bayernline.de
    last modified        : $Date: 2009/11/01 20:40:38 $
                           $Revision: 1.82 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 This file provides an user interface to change a virtual address
 ***************************************************************************/

#include <QLabel>
#include <QLayout>
#include <QMessageBox>
#include <QDialogButtonBox>

#include "virtualaddressdialog.h"
#include "preferences.h"
#include "resources.h"
#include "srcpmessage.h"

enum {
    MINMMPORT = 0,
    MAXMMPORT = 1,
    MINDCCPORT = 0,
    MAXDCCPORT = 1,
    MINSVPORT = 0,
    MAXSVPORT = 64535, // no limits
    MINSXPORT = 1,
    MAXSXPORT = 8
};



VirtualAddressDialog::VirtualAddressDialog(QWidget* parent):
    QDialog(parent)
{
    setObjectName("VirtualAddressDialog");
    setWindowTitle(tr("Edit address"));
    classid = element::siciNone;

    /*Layout to separate OK Cancel buttons from the upper rest*/
    QBoxLayout* baseLayout = new QVBoxLayout(this);
    baseLayout->setSpacing(10);

    /*right decoder data group box*/
    QGroupBox* decoderGB = new QGroupBox(tr("Virtual address"));
    decoderGB->setObjectName("decoderGB");
    baseLayout->addWidget(decoderGB);
    QVBoxLayout* decoderGBL = new QVBoxLayout();
    decoderGB->setLayout(decoderGBL);

    QGridLayout* decdataLayout = new QGridLayout();
    decdataLayout->setObjectName("decdataLayout");
    decoderGBL->addLayout(decdataLayout);

    /*line with srcp bus 1 */
    QLabel* srcpBus1Label = new QLabel(tr("S&RCP-Bus:"), decoderGB);
    decdataLayout->addWidget(srcpBus1Label, 0, 0);

    srcpBus1LE = new QLineEdit(decoderGB);
    srcpBus1LE->setObjectName("srcpBus1LE");
    srcpBus1LE->setMaxLength(4);
    srcpBus1LE->setMaximumWidth(LEMAXWIDTH);
    decdataLayout->addWidget(srcpBus1LE, 0, 1);
    srcpBus1Label->setBuddy(srcpBus1LE);

    /*line with address 1 */
    QLabel* address1Lbl = new QLabel(tr("A&ddress:"), decoderGB);
    decdataLayout->addWidget(address1Lbl, 1, 0);

    address1LE = new QLineEdit(decoderGB);
    address1LE->setObjectName("address_1");
    decdataLayout->addWidget(address1LE, 1, 1);
    address1LE->setMaxLength(4);       // address length of NA protocol
    address1LE->setMaximumWidth(LEMAXWIDTH);
    addressVdt = new QIntValidator(0, MAX_GADCC, this);
    address1LE->setValidator(addressVdt);
    address1Lbl->setBuddy(address1LE);

    /*spacer to push contents of box to top */
    decoderGBL->addStretch();


    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok |
            QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(validate()));
    connect(buttonBox, SIGNAL(rejected()), this,
            SLOT(reject()));
    baseLayout->addWidget(buttonBox);
}


int VirtualAddressDialog::getSRCPBus1()
{
    return srcpBus1LE->text().toInt();
}


void VirtualAddressDialog::setSRCPBus1(int bus)
{
    srcpBus1LE->setText(QString::number(bus));
}


int VirtualAddressDialog::getAddress1()
{
    return address1LE->text().toInt();
}


void VirtualAddressDialog::setAddress1(int addr)
{
    address1LE->setText(QString::number(addr));
}


void VirtualAddressDialog::updateValidators()
{
    /*virtual address ranges*/
    if (classid == element::siciZt1 || classid == element::siciZt3 ||
            classid == element::siciRt1 || classid == element::siciRt3) {
        addressVdt->setTop(MAX_RB);
        addressVdt->setBottom(MIN_RB);
    }
    else if (classid == element::siciAdr) {
        addressVdt->setTop(MAX_DISP);
        addressVdt->setBottom(MIN_DISP);
    }

    else if (classid == element::siciKrh || classid == element::siciKr1
            || classid == element::siciKl1) {
        addressVdt->setTop(MAX_CROSS);
        addressVdt->setBottom(MIN_CROSS);
    }
}

void VirtualAddressDialog::updateAddressTooltip()
{
    address1LE->setToolTip(tr(
                "Enter address of decoder 1.\n"
                "Valid range is %1..%2.")
            .arg(addressVdt->bottom())
            .arg(addressVdt->top()));
}

void VirtualAddressDialog::setClassId(element::SpdrItemClassId ci)
{
    classid = ci;
    updateValidators();
    updateAddressTooltip();
}


void VirtualAddressDialog::validate()
{
    if (address1LE->isEnabled() && !address1LE->hasAcceptableInput()) {
        address1LE->setFocus();
        address1LE->selectAll();
        QMessageBox::warning(this, tr("Unvalid address detected"),
                tr("Value of address is not valid.\n"
                    "Valid range is %1..%2.")
                .arg(addressVdt->bottom())
                .arg(addressVdt->top())
        , QMessageBox::Ok);
        return;
    }
    accept();
}
